@@ -22,11 +22,17 @@
 #include "common/str_compat.hpp"
 #include "common/utils/string_utils.hpp"
 
-//#define BUFFER_TIME 120000
-#define PERIOD_TIME 15000
+using namespace std::chrono_literals;
+using namespace std;
+
+namespace player
+{
+
+static constexpr std::chrono::milliseconds BUFFER_TIME = 80ms;
+static constexpr int PERIODS = 4;
+
 #define exp10(x) (exp((x)*log(10)))
 
-using namespace std;
 
 static constexpr auto LOG_TAG = "Alsa";
 static constexpr auto DEFAULT_MIXER = "PCM";
@@ -61,6 +67,16 @@ AlsaPlayer::AlsaPlayer(boost::asio::io_context& io_context, const ClientSettings
 
         LOG(DEBUG, LOG_TAG) << "Mixer: " << mixer_name_ << ", device: " << mixer_device_ << "\n";
     }
+
+    buffer_time_ = BUFFER_TIME;
+    periods_ = PERIODS;
+    auto params = utils::string::split_pairs(settings.parameter, ',', '=');
+    if (params.find("buffer_time") != params.end())
+        buffer_time_ = std::chrono::milliseconds(std::max(cpt::stoi(params["buffer_time"]), 10));
+    if (params.find("fragments") != params.end())
+        periods_ = std::max(cpt::stoi(params["fragments"]), 2);
+
+    LOG(INFO, LOG_TAG) << "Using buffer_time: " << buffer_time_.count() / 1000 << " ms, fragments: " << periods_ << "\n";
 }
 
 
@@ -261,30 +277,28 @@ void AlsaPlayer::initMixer()
 void AlsaPlayer::initAlsa()
 {
     std::lock_guard<std::recursive_mutex> lock(mutex_);
-    unsigned int tmp, rate;
-    int err, channels;
-    snd_pcm_hw_params_t* params;
 
     const SampleFormat& format = stream_->getFormat();
-    rate = format.rate();
-    channels = format.channels();
+    unsigned int rate = format.rate();
+    int channels = format.channels();
+    int err;
 
-    /* Open the PCM device in playback mode */
+    // Open the PCM device in playback mode
     if ((err = snd_pcm_open(&handle_, settings_.pcm_device.name.c_str(), SND_PCM_STREAM_PLAYBACK, 0)) < 0)
         throw SnapException("Can't open " + settings_.pcm_device.name + ", error: " + snd_strerror(err), err);
 
-    /*	struct snd_pcm_playback_info_t pinfo;
-     if ( (pcm = snd_pcm_playback_info( pcm_handle, &pinfo )) < 0 )
-     fprintf( stderr, "Error: playback info error: %s\n", snd_strerror( err ) );
-     printf("buffer: '%d'\n", pinfo.buffer_size);
-     */
-    /* Allocate parameters object and fill it with default values*/
-    snd_pcm_hw_params_alloca(&params);
+    // struct snd_pcm_playback_info_t pinfo;
+    // if ((pcm = snd_pcm_playback_info( pcm_handle, &pinfo)) < 0)
+    //     fprintf(stderr, "Error: playback info error: %s\n", snd_strerror(err));
+    // printf("buffer: '%d'\n", pinfo.buffer_size);
 
+    // Allocate parameters object and fill it with default values
+    snd_pcm_hw_params_t* params;
+    snd_pcm_hw_params_alloca(&params);
     if ((err = snd_pcm_hw_params_any(handle_, params)) < 0)
         throw SnapException("Can't fill params: " + string(snd_strerror(err)));
 
-    /* Set parameters */
+    // Set parameters
     if ((err = snd_pcm_hw_params_set_access(handle_, params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0)
         throw SnapException("Can't set interleaved mode: " + string(snd_strerror(err)));
 
@@ -333,41 +347,34 @@ void AlsaPlayer::initAlsa()
 
     if ((err = snd_pcm_hw_params_set_rate_near(handle_, params, &rate, nullptr)) < 0)
         throw SnapException("Can't set rate: " + string(snd_strerror(err)));
+    if (rate != format.rate())
+        LOG(WARNING, LOG_TAG) << "Could not set sample rate to " << format.rate() << " Hz, using: " << rate << " Hz\n";
 
-    unsigned int period_time;
-    snd_pcm_hw_params_get_period_time_max(params, &period_time, nullptr);
-    if (period_time > PERIOD_TIME)
-        period_time = PERIOD_TIME;
+    unsigned int buffer_time = buffer_time_.count();
+    if ((err = snd_pcm_hw_params_set_buffer_time_near(handle_, params, &buffer_time, nullptr)) < 0)
+        throw SnapException("Can't set buffer time: " + string(snd_strerror(err)));
 
-    unsigned int buffer_time = 4 * period_time;
-
-    snd_pcm_hw_params_set_period_time_near(handle_, params, &period_time, nullptr);
-    snd_pcm_hw_params_set_buffer_time_near(handle_, params, &buffer_time, nullptr);
+    unsigned int periods = periods_;
+    if ((err = snd_pcm_hw_params_set_periods_near(handle_, params, &periods, 0)) < 0)
+        throw SnapException("Can't set periods: " + string(snd_strerror(err)));
 
     //	long unsigned int periodsize = stream_->format.msRate() * 50;//2*rate/50;
     //	if ((pcm = snd_pcm_hw_params_set_buffer_size_near(pcm_handle, params, &periodsize)) < 0)
     //		LOG(ERROR, LOG_TAG) << "Unable to set buffer size " << (long int)periodsize << ": " <<  snd_strerror(pcm) << "\n";
 
-    /* Write parameters */
+    // Write parameters
     if ((err = snd_pcm_hw_params(handle_, params)) < 0)
         throw SnapException("Can't set hardware parameters: " + string(snd_strerror(err)));
 
-    /* Resume information */
-    LOG(DEBUG, LOG_TAG) << "PCM name: " << snd_pcm_name(handle_) << "\n";
-    LOG(DEBUG, LOG_TAG) << "PCM state: " << snd_pcm_state_name(snd_pcm_state(handle_)) << "\n";
-    snd_pcm_hw_params_get_channels(params, &tmp);
-    LOG(DEBUG, LOG_TAG) << "channels: " << tmp << "\n";
-
-    snd_pcm_hw_params_get_rate(params, &tmp, nullptr);
-    LOG(DEBUG, LOG_TAG) << "rate: " << tmp << " bps\n";
-
-    /* Allocate buffer to hold single period */
+    // Resume information
+    unsigned int period_time;
+    snd_pcm_hw_params_get_period_time(params, &period_time, nullptr);
     snd_pcm_hw_params_get_period_size(params, &frames_, nullptr);
-    LOG(DEBUG, LOG_TAG) << "frames: " << frames_ << "\n";
-
-    snd_pcm_hw_params_get_period_time(params, &tmp, nullptr);
-    LOG(DEBUG, LOG_TAG) << "period time: " << tmp << "\n";
+    LOG(INFO, LOG_TAG) << "PCM name: " << snd_pcm_name(handle_) << ", sample rate: " << rate << " Hz, channels: " << channels
+                       << ", buffer time: " << buffer_time << " us, periods: " << periods << ", period time: " << period_time
+                       << " us, period frames: " << frames_ << "\n";
 
+    // Allocate buffer to hold single period
     snd_pcm_sw_params_t* swparams;
     snd_pcm_sw_params_alloca(&swparams);
     snd_pcm_sw_params_current(handle_, swparams);
@@ -377,6 +384,12 @@ void AlsaPlayer::initAlsa()
     //	snd_pcm_sw_params_set_stop_threshold(pcm_handle, swparams, frames_);
     snd_pcm_sw_params(handle_, swparams);
 
+    if (snd_pcm_state(handle_) == SND_PCM_STATE_PREPARED)
+    {
+        if ((err = snd_pcm_start(handle_)) < 0)
+            LOG(DEBUG, LOG_TAG) << "Failed to start PCM: " << snd_strerror(err) << "\n";
+    }
+
     if (ctl_ == nullptr)
         initMixer();
 }
@@ -461,6 +474,35 @@ bool AlsaPlayer::needsThread() const
 }
 
 
+bool AlsaPlayer::getAvailDelay(snd_pcm_sframes_t& avail, snd_pcm_sframes_t& delay)
+{
+    int result = snd_pcm_avail_delay(handle_, &avail, &delay);
+    if (result < 0)
+    {
+        LOG(WARNING, LOG_TAG) << "snd_pcm_avail_delay failed: " << snd_strerror(result) << " (" << result << "), avail: " << avail << ", delay: " << delay
+                              << ", using snd_pcm_avail amd snd_pcm_delay.\n";
+        this_thread::sleep_for(1ms);
+        avail = snd_pcm_avail(handle_);
+        result = snd_pcm_delay(handle_, &delay);
+        if ((result < 0) || (delay < 0))
+        {
+            LOG(WARNING, LOG_TAG) << "snd_pcm_delay failed: " << snd_strerror(result) << " (" << result << "), avail: " << avail << ", delay: " << delay
+                                  << "\n";
+            return false;
+        }
+        // LOG(DEBUG, LOG_TAG) << "snd_pcm_delay: " << delay << ", snd_pcm_avail: " << avail << "\n";
+    }
+
+    if (avail < 0)
+    {
+        LOG(DEBUG, LOG_TAG) << "snd_pcm_avail failed: " << snd_strerror(avail) << " (" << avail << "), using " << frames_ << "\n";
+        avail = frames_;
+    }
+
+    return true;
+}
+
+
 void AlsaPlayer::worker()
 {
     snd_pcm_sframes_t pcm;
@@ -491,7 +533,7 @@ void AlsaPlayer::worker()
         int wait_result = snd_pcm_wait(handle_, 100);
         if (wait_result == -EPIPE)
         {
-            LOG(ERROR, LOG_TAG) << "XRUN: " << snd_strerror(wait_result) << "\n";
+            LOG(ERROR, LOG_TAG) << "XRUN while waiting for PCM: " << snd_strerror(wait_result) << "\n";
             snd_pcm_prepare(handle_);
         }
         else if (wait_result < 0)
@@ -505,38 +547,16 @@ void AlsaPlayer::worker()
             continue;
         }
 
-        int result = snd_pcm_avail_delay(handle_, &framesAvail, &framesDelay);
-        if (result < 0)
+        if (!getAvailDelay(framesAvail, framesDelay))
         {
-            // if (result == -EPIPE)
-            //     snd_pcm_prepare(handle_);
-            // else
-            //     uninitAlsa();
-            LOG(WARNING, LOG_TAG) << "snd_pcm_avail_delay failed: " << snd_strerror(result) << ", avail: " << framesAvail << ", delay: " << framesDelay
-                                  << ", retrying.\n";
-            this_thread::sleep_for(5ms);
-            int result = snd_pcm_avail_delay(handle_, &framesAvail, &framesDelay);
-            if (result < 0)
-            {
-                this_thread::sleep_for(5ms);
-                LOG(WARNING, LOG_TAG) << "snd_pcm_avail_delay failed again: " << snd_strerror(result) << ", avail: " << framesAvail
-                                      << ", delay: " << framesDelay << ", using snd_pcm_avail and snd_pcm_delay.\n";
-                framesAvail = snd_pcm_avail(handle_);
-                result = snd_pcm_delay(handle_, &framesDelay);
-                if ((result < 0) || (framesAvail <= 0) || (framesDelay <= 0))
-                {
-                    LOG(WARNING, LOG_TAG) << "snd_pcm_avail and snd_pcm_delay failed: " << snd_strerror(result) << ", avail: " << framesAvail
-                                          << ", delay: " << framesDelay << "\n";
-                    this_thread::sleep_for(10ms);
-                    snd_pcm_prepare(handle_);
-                    continue;
-                }
-            }
+            this_thread::sleep_for(10ms);
+            snd_pcm_prepare(handle_);
+            continue;
         }
 
         if (framesAvail < static_cast<snd_pcm_sframes_t>(frames_))
         {
-            this_thread::sleep_for(10ms);
+            this_thread::sleep_for(5ms);
             continue;
         }
 
@@ -555,7 +575,7 @@ void AlsaPlayer::worker()
             adjustVolume(buffer_.data(), framesAvail);
             if ((pcm = snd_pcm_writei(handle_, buffer_.data(), framesAvail)) == -EPIPE)
             {
-                LOG(ERROR, LOG_TAG) << "XRUN: " << snd_strerror(pcm) << "\n";
+                LOG(ERROR, LOG_TAG) << "XRUN while writing to PCM: " << snd_strerror(pcm) << "\n";
                 snd_pcm_prepare(handle_);
             }
             else if (pcm < 0)
@@ -625,3 +645,5 @@ vector<PcmDevice> AlsaPlayer::pcm_list()
     snd_device_name_free_hint(hints);
     return result;
 }
+
+} // namespace player
